<?php
class gLastFM_Widget extends WP_Widget
{
	private $fromcache = FALSE;
	private $defaults;

	/*
	 * Constructor of the widget
	 */
	public function gLastFM_Widget()
	{
		$this->defaults = array
		(
			'title'           => 'Last.fm',
			'type'            => 'recenttracks',
			'limit'           => 5,
			'use_template'    => FALSE,
			'display_credits' => TRUE,
			'caching_message' => TRUE,
			'image_size'      => 2,
			'template'        => <<<TEMPLATE
<div class="info">
<p>
<span class="piste">
%iftrackurl%
</span>
<span class="time">
%date%
</span>
</p>
<p class="artiste">
%ifartisturl%
</p>
</div>
TEMPLATE
		);

		/* Widget settings. */
		$widget_ops = array
		(
			'classname' => 'gLastFM_container',
			'description' => __( 'A widget that displays recent, top or loved tracks from Last.fm', GLASTFM_TEXT_DOMAIN )
		);

		/* Widget control settings. */
		$control_ops = array
		(
			'width'   => 200,
			'height'  => 350,
			'id_base' => 'glastfm-widget'
		);

		/* Create the widget. */
		parent::WP_Widget('glastfm-widget', __( 'gLastFM Widget' ), $widget_ops, $control_ops );
	}

	/*
	 * Retrieve the JSON data from Last.fm or from the cache
	 */
	public function glastfm_getdata( $url, $username, $type )
	{
		$cachedFile = GLASTFM_PLUGIN_DIR . "cache/{$username}-{$type}.json";

		/* Create the cached file if it doesn't exist */
		if( ! file_exists( $cachedFile ) )
		{
			$fp = fopen( $cachedFile, 'w' );
			fclose( $fp );
		}

		/* get the content of the cached file */
		$cachedData = file_get_contents( $cachedFile );

		/* if cached file is less than 2 minutes old, return the content of the cached file */
		if( ( time() - filemtime( $cachedFile ) < 120 ) && ! empty( $cachedData ) )
		{
			$this->fromcache = TRUE;
			return json_decode( $cachedData, TRUE );
		}
		else /* get content from Last.fm and save it to cache file */
		{
			$curl = curl_init();
			curl_setopt( $curl, CURLOPT_URL, $url );
			curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true );
			curl_setopt( $curl, CURLOPT_TIMEOUT, 2 );
			$freshData = curl_exec( $curl );

			$json = json_decode( $freshData, TRUE );
			if( empty( $json ) || ! empty( $json['error'] ) )
			{
				/* get the content of the cached file instead of the error from Last.fm
				 * (only if the cached file is not empty, otherwise we still get the Last.fm error)
				 */
				if( ! empty( $cachedData ) )
				{
					$this->fromcache = TRUE;
					$json = json_decode( $cachedData, TRUE );
				}
			}
			else
			{
				/* update the cached file with fresh data */
				file_put_contents( $cachedFile, $freshData );
			}
			return $json;
		}
	}


	/*
	 * Renders the widget on a sidebar
	 */
	public function widget( $args, $instance )
	{
		// If no API Key entered
		if( get_option( 'glastfm_api', 'no' ) == 'no' )
		{
			_e( 'No API key entered, you have to configure gLastFM settings', GLASTFM_TEXT_DOMAIN );
		}

		extract( $args );

		/* Set up some default widget settings. */
		$instance = wp_parse_args( (array) $instance, $this->defaults );

		$title           = apply_filters( 'widget_title', $instance['title'] );
		$username        = $instance['user_id'];
		$limit           = $instance['limit'];
		$api_key         = $instance['api_key'];
		$type            = $instance['type'];
		$use_template    = $instance['use_template'];
		$template        = $instance['template'];
		$display_credits = $instance['display_credits'];
		$caching_message = $instance['caching_message'];
		$image_size      = $instance['image_size'];

		// Fetch the XML data from Last.fm API server
		$response = $this->glastfm_getdata
		(
			"http://ws.audioscrobbler.com/2.0/?method=user.get{$type}&limit={$limit}&user={$username}&format=json&api_key=" . get_option( 'glastfm_api' ),
			$username,
			$type
		);
		if( empty( $response ) )
		{
			return FALSE;
		}

		// Before widget (defined by themes).
		echo $before_widget;
		?>

		<div class="gLastFM">

			<?php
			// Widget Title
			if( $title )
			{
				echo $before_title . "<a href=\"http://www.last.fm/user/{$username}\">{$title}</a>{$after_title}";
			}
			// The funny part begins
			$i = 0;
			// Debug to check the XML format
			//echo "<pre>".print_r($xml,true)."</pre>";

			if( ! empty( $response['error'] ) )
			{
				$why = $response['message'];
				$errorcode = $response['error'];
				echo '<p>' . __( 'Unable to retrieve data from Last.fm', GLASTFM_TEXT_DOMAIN ) . '</p><p>' . __( 'Please try again later!', GLASTFM_TEXT_DOMAIN ) . '</p>';
				echo '<p>' . __( 'Why: ', GLASTFM_TEXT_DOMAIN ) . $why . '</p>';
				echo '<p>' . __( 'Code: ', GLASTFM_TEXT_DOMAIN ) . $errorcode . '</p>';
			}
			else
			{
				?>

				<ul class="tracks">

				<?php

				//define the element name
				$elements = Array
				(
					'recenttracks'      => 'track',
					'lovedtracks'       => 'track',
					'toptracks'         => 'track',
					'topartists'        => 'artist',
					'topalbums'         => 'album',
					'weeklytrackchart'  => 'track',
					'weeklyalbumchart'  => 'album',
					'weeklyartistchart' => 'artist'
				);

				foreach( $response[$type][$elements[$type]] as $key => $v )
				{
					//artist name
					$artist    = __( '(unknown)', GLASTFM_TEXT_DOMAIN );
					$artisturl = '#';

					if( isset( $v['artist']['#text'] ) )
					{
						$artist = $v['artist']['#text'];
					}
					elseif( isset( $v['artist']['name'] ) )
					{
						$artist = $v['artist']['name'];
					}
					elseif( in_array( $type, array( 'topartists', 'weeklyartistchart' ) ) && isset( $v['name'] ) )
					{
						$artist = $v['name'];
					}

					//artist url
					if( isset( $v['artist']['url'] ) )
					{
						$artisturl = $v['artist']['url'];
					}
					elseif( in_array( $type, array( 'topartists', 'weeklyartistchart' ) ) && isset( $v['url'] ) )
					{
						$artisturl = $v['url'];
					}
					if( empty( $artisturl ) )
					{
						$artisturl = '#';
					}

					//track url
					$trackurl  = '#';
					if( in_array( $type, array( 'recenttracks', 'lovedtracks', 'toptracks', 'weeklytrackchart' ) ) && isset( $v['url'] ) )
					{
						$trackurl = $v['url'];
						if( empty( $trackurl ) )
						{
							$trackurl = '#';
						}
					}

					//album url
					$albumurl  = '#';
					if( in_array( $type, array( 'topalbums', 'weeklyalbumchart' ) ) && isset( $v['url'] ) )
					{
						$albumurl = $v['url'];
						if( empty( $albumurl ) )
						{
							$albumurl = '#';
						}
					}

					//track title
					$track  = __( '(unknown)', GLASTFM_TEXT_DOMAIN );
					if( in_array( $type, array( 'recenttracks', 'lovedtracks', 'toptracks', 'weeklytrackchart' ) ) && isset( $v['name'] ) )
					{
						$track = $v['name'];
					}

					//play count
					$playcount  = __( 'N/A', GLASTFM_TEXT_DOMAIN );
					if( isset( $v['playcount'] ) )
					{
						$playcount = $v['playcount'];
					}

					//album name
					$album  = __( '(unknown)', GLASTFM_TEXT_DOMAIN );
					if( isset( $v['album']['#text'] ) )
					{
						$album = $v['album']['#text'];
					}
					elseif( in_array( $type, array( 'topalbums', 'weeklyalbumchart' ) ) && isset( $v['name'] ) )
					{
						$album = $v['name'];
					}

					//date
					$date_display = __( 'N/A', GLASTFM_TEXT_DOMAIN );
					if( isset( $v['date']['uts'] ) )
					{
						$decal = ( substr( date( 'O' ), 0, 1 ) == '-' ? -1 : +1 ) * intval( substr( date( 'O' ), -4, 4 ) ) / 100;
						$duree = abs( time() - intval( $v['date']['uts'] ) - $decal * 3600 );
						if( $duree < 60 )
						{
							$unit = $duree == 1 ? __( 'sec', GLASTFM_TEXT_DOMAIN) : __( 'secs', GLASTFM_TEXT_DOMAIN );
						}
						elseif( $duree <= 3600 )
						{
							$duree = round( $duree / 60 );
							$unit  = $duree == 1 ? __( 'min', GLASTFM_TEXT_DOMAIN ) : __( 'mins', GLASTFM_TEXT_DOMAIN );
						}
						elseif( $duree <= 86400 )
						{
							$duree= round( $duree / 3600 );
							$unit = $duree == 1 ? __( 'hour', GLASTFM_TEXT_DOMAIN ) : __( 'hours', GLASTFM_TEXT_DOMAIN );
						}
						else
						{
							$duree = round( $duree / 86400 );
							$unit  = $duree == 1 ? __( 'day', GLASTFM_TEXT_DOMAIN ) : __( 'days', GLASTFM_TEXT_DOMAIN );
						}
						$date_display = sprintf( __( '%d %s ago', GLASTFM_TEXT_DOMAIN ), $duree, $unit );
					}
					elseif( isset( $v['@attr']['nowplaying'] ) && $v['@attr']['nowplaying'] == TRUE )
					{
						$date_display = __( 'Now playing', GLASTFM_TEXT_DOMAIN );
					}

					//cover url
					$picurl = GLASTFM_PLUGIN_URL . 'theme/empty_' . $image_size . '.png';
					if( ! empty( $v['image'][intval( $image_size )]['#text'] ) )
					{
						$picurl            = $v['image'][intval( $image_size )]['#text'];
						$picurl_small      = $v['image'][0]['#text'];
						$picurl_medium     = $v['image'][1]['#text'];
						$picurl_large      = $v['image'][2]['#text'];
						$picurl_extralarge = $v['image'][3]['#text'];
					}

					//begin to output each item
					?>

					<li class="track <?php echo ( $use_template ? 'template' : 'notemplate' ); ?> size_<?php echo $image_size; ?>" id="track-<?php echo $i + 1; ?>">

						<?php if( ! $use_template ): ?>

							<div class="CDCase">
								<?php
								if( $trackurl != '#' )
								{
									?>

									<a href="<?php echo $trackurl; ?>">

									<?php
								}

								echo "<img src=\"{$picurl}\" alt=\"" . __( 'Click to view the Last.fm page for this track', GLASTFM_TEXT_DOMAIN ) . '" />';

								if( $trackurl != '#' )
								{
									echo '</a>';
								}
								echo "\n";
								?>

							</div>
							<div class="info">

								<?php if( in_array( $type, array( 'recenttracks', 'lovedtracks', 'toptracks', 'weeklytrackchart' ) ) ): ?>

									<span class="piste"><?php   echo ( $trackurl  != '#' ? '<a href="' . $trackurl  . '">' . $track  . '</a>' : $track ); ?></span>

									<?php if( $image_size == 0 || $image_size == 3 ): ?>

										<span> - </span>

									<?php else: ?>

										<br />

									<?php endif; ?>

								<?php endif; ?>

								<span class="artiste"><?php echo ( $artisturl != '#' ? '<a href="' . $artisturl . '">' . $artist . '</a>' : $artist ); ?></span>

								<br />

								<?php if( in_array( $type, array( 'recenttracks', 'topalbums', 'weeklyalbumchart' ) ) ): ?>

									<span class="album"><?php echo ( $albumurl != '#' ? '<a href="' . $albumurl . '">' . $album . '</a>' : $album ); ?></span>

									<?php if( $image_size == 0 || $image_size == 3 ): ?>

										<span> - </span>

									<?php else: ?>

										<br />

									<?php endif; ?>

								<?php endif; ?>
								<?php if( in_array( $type, array( 'recenttracks' ) ) ): ?>

									<span class="time"><?php echo $date_display; ?></span>

								<?php endif; ?>
								<?php if( in_array( $type, array( 'toptracks', 'topartists', 'topalbums', 'weeklytrackchart', 'weeklyalbumchart', 'weeklyartistchart' ) ) ): ?>

									<span class="playcount"><?php printf( __( 'Played %d times', GLASTFM_TEXT_DOMAIN ), $playcount ); ?></span>

								<?php endif; ?>

							</div><!-- info -->

						<?php else: ?>

							<?php

							$display = $template;
							$display = str_replace( '%trackurl%',   $trackurl,     $display );
							$display = str_replace( '%albumurl%',   $albumurl,     $display );
							$display = str_replace( '%artisturl%',  $artisturl,    $display );
							$display = str_replace( '%coverurl%',   $picurl,       $display );
							$display = str_replace( '%coversmallurl%',   $picurl_small,   $display );
							$display = str_replace( '%covermediumurl%',   $picurl_medium,   $display );
							$display = str_replace( '%coverlargeurl%',     $picurl_large,     $display );
							$display = str_replace( '%coverextralargeurl%', $picurl_extralarge, $display );
							$display = str_replace( '%artist%',     $artist,       $display );
							$display = str_replace( '%album%',      $album,        $display );
							$display = str_replace( '%track%',      $track,        $display );
							$display = str_replace( '%count%',      $playcount,    $display );
							$display = str_replace( '%date%',       $date_display, $display );
							$display = str_replace( '%iftrackurl%',  $trackurl  != '#' ? '<a href="' . $trackurl  . '">' . $track  . '</a>' : $track,  $display );
							$display = str_replace( '%ifalbumurl%',  $albumurl  != '#' ? '<a href="' . $albumurl . '">'  . $album  . '</a>' : $album,  $display );
							$display = str_replace( '%ifartisturl%', $artisturl != '#' ? '<a href="' . $artisturl . '">' . $artist . '</a>' : $artist, $display );

							echo $display;

							?>

						<?php endif; ?>

					</li><!-- track -->

					<?php
					$i++;

					//weekly charts do not have a limit parameter, and recent tracks "now playing" do not respect the limit paramter, so we simulate it.
					if( $i >= $limit )
					{
						break;
					}
				}
				?>

				</ul>

				<?php
			}
			if( $this->fromcache == TRUE && $caching_message == TRUE )
			{
				echo '<p><small>' . __( '(from cache)', GLASTFM_TEXT_DOMAIN ) . "</small></p>\n";
			}
			if( $display_credits == TRUE )
			{
				?>
				<p>
					<small>
						<?php
						printf
						(
							__( '<a href="%s">Last.fm widget</a> by <a href="">GabSoftware</a>', GLASTFM_TEXT_DOMAIN ),
							'http://www.gabsoftware.com/products/scripts/glastfm/',
							'http://www.gabsoftware.com/'
						); ?>
					</small>
				</p>
			<?php
			}
			?>

		</div>

		<?php
		echo $after_widget;
	}

	/*
	 * Called after we click on the Save button
	 */
	public function update( $new_instance, $old_instance )
	{
		$instance = $old_instance;

		/* Strip tags (if needed) and update the widget settings. */
		$instance['title']           = strip_tags( $new_instance['title'] );
		$instance['user_id']         = $new_instance['user_id'];
		$instance['limit']           = $new_instance['limit'];
		$instance['api_key']         = $new_instance['api_key'];
		$instance['type']            = $new_instance['type'];
		$instance['use_template']    = ! empty( $new_instance['use_template'] );
		$instance['display_credits'] = ! empty( $new_instance['display_credits'] );
		$instance['caching_message'] = ! empty( $new_instance['caching_message'] );
		$instance['template']        = $new_instance['template'];
		$instance['image_size']      = $new_instance['image_size'];

		return $instance;
	}

	/*
	 * Renders the widget form in the dashboard
	 */
	public function form( $instance )
	{
		/* Set up some default widget settings. */
		$instance = wp_parse_args( (array) $instance, $this->defaults );
		$options  = array
		(
			'recenttracks'      => __( 'Recent tracks',       GLASTFM_TEXT_DOMAIN ),
			'lovedtracks'       => __( 'Loved tracks',        GLASTFM_TEXT_DOMAIN ),
			'toptracks'         => __( 'Top tracks',          GLASTFM_TEXT_DOMAIN ),
			'topartists'        => __( 'Top artists',         GLASTFM_TEXT_DOMAIN ),
			'topalbums'         => __( 'Top albums',          GLASTFM_TEXT_DOMAIN ),
			'weeklytrackchart'  => __( 'Weekly track chart',  GLASTFM_TEXT_DOMAIN ),
			'weeklyalbumchart'  => __( 'Weekly album chart',  GLASTFM_TEXT_DOMAIN ),
			'weeklyartistchart' => __( 'Weekly artist chart', GLASTFM_TEXT_DOMAIN )
		);

		$image_sizes  = array
		(
			0 => __( 'Small (34x34)',         GLASTFM_TEXT_DOMAIN ),
			1 => __( 'Medium (64x64)',        GLASTFM_TEXT_DOMAIN ),
			2 => __( 'Large (126x126)',       GLASTFM_TEXT_DOMAIN ),
			3 => __( 'Extra large (300x300)', GLASTFM_TEXT_DOMAIN )
		);
		?>

		<p>
			<label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e( 'Title:', GLASTFM_TEXT_DOMAIN ); ?></label>
			<input id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" value="<?php echo $instance['title']; ?>" style="width:100%;" />
		</p>

		<p>
			<label for="<?php echo $this->get_field_id( 'user_id' ); ?>"><?php _e( 'Last.fm username:', GLASTFM_TEXT_DOMAIN ); ?></label>
			<input id="<?php echo $this->get_field_id( 'user_id' ); ?>" name="<?php echo $this->get_field_name( 'user_id' ); ?>" value="<?php echo $instance['user_id']; ?>" style="width:80%;" />
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'limit' ); ?>"><?php _e( 'How many items:', GLASTFM_TEXT_DOMAIN ); ?></label>
			<input id="<?php echo $this->get_field_id( 'limit' ); ?>" name="<?php echo $this->get_field_name( 'limit' ); ?>" value="<?php echo $instance['limit']; ?>" style="width:2em;" />
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'type' ); ?>"><?php _e( 'What do you want to display?', GLASTFM_TEXT_DOMAIN ); ?></label>
			<select id="<?php echo $this->get_field_id( 'type' ); ?>" name="<?php echo $this->get_field_name( 'type' ); ?>">

				<?php foreach( $options as $o => $v ): ?>

					<option value="<?php echo $o; ?>" <?php if( $instance['type'] == $o ) echo 'selected="selected"'; ?>><?php echo $v; ?></option>

				<?php endforeach; ?>

			</select>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'image_size' ); ?>"><?php _e( 'Cover size:', GLASTFM_TEXT_DOMAIN ); ?></label>
			<select id="<?php echo $this->get_field_id( 'image_size' ); ?>" name="<?php echo $this->get_field_name( 'image_size' ); ?>">

				<?php foreach( $image_sizes as $o => $v ): ?>

					<option value="<?php echo $o; ?>" <?php if( $instance['image_size'] == $o ) echo 'selected="selected"'; ?>><?php echo $v; ?></option>

				<?php endforeach; ?>

			</select>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'use_template' ); ?>"><?php _e( 'Use template:', GLASTFM_TEXT_DOMAIN ); ?></label>
			<input type="checkbox" id="<?php echo $this->get_field_id( 'use_template' ); ?>" name="<?php echo $this->get_field_name( 'use_template' ); ?>" value="on"<?php echo ( $instance['use_template'] === TRUE ? ' checked="checked"' : '' ); ?> style="width:2em;" />
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'template' ); ?>"><?php _e( 'Template for each item:', GLASTFM_TEXT_DOMAIN ); ?></label><br />
			<textarea id="<?php echo $this->get_field_id( 'template' ); ?>" name="<?php echo $this->get_field_name( 'template' ); ?>" cols="35" rows="8"><?php echo $instance['template']; ?></textarea>
		</p>
		<p><?php echo __( 'The following tags are available:', GLASTFM_TEXT_DOMAIN ) ; ?></p>
		<dl>
			<dt><strong>%track%      </strong></dt><dd><?php _e( 'Track title <sup>(1)</sup>', GLASTFM_TEXT_DOMAIN ); ?></dd>
			<dt><strong>%trackurl%   </strong></dt><dd><?php _e( 'Track URL <sup>(1)</sup>',   GLASTFM_TEXT_DOMAIN ); ?></dd>

			<dt><strong>%album%      </strong></dt><dd><?php _e( 'Album name <sup>(2)</sup>',  GLASTFM_TEXT_DOMAIN ); ?></dd>
			<dt><strong>%albumurl%   </strong></dt><dd><?php _e( 'Album URL <sup>(3)</sup>',   GLASTFM_TEXT_DOMAIN ); ?></dd>

			<dt><strong>%artist%     </strong></dt><dd><?php _e( 'Artist name',                GLASTFM_TEXT_DOMAIN ); ?></dd>
			<dt><strong>%artisturl%  </strong></dt><dd><?php _e( 'Artist URL <sup>(4)</sup>',  GLASTFM_TEXT_DOMAIN ); ?></dd>

			<dt><strong>%date%       </strong></dt><dd><?php _e( 'Date played <sup>(5)</sup>', GLASTFM_TEXT_DOMAIN ); ?></dd>
			<dt><strong>%count%      </strong></dt><dd><?php _e( 'Play count <sup>(6)</sup>',  GLASTFM_TEXT_DOMAIN ); ?></dd>

			<dt><strong>%iftrackurl% </strong></dt><dd><?php _e( 'Displays a link to the track URL if possible, otherwise displays the track title <sup>(1)</sup>',    GLASTFM_TEXT_DOMAIN ); ?></dd>
			<dt><strong>%ifalbumurl% </strong></dt><dd><?php _e( 'Displays a link to the album URL if possible, otherwise displays the album title <sup>(3, 7)</sup>', GLASTFM_TEXT_DOMAIN ); ?></dd>
			<dt><strong>%ifartisturl%</strong></dt><dd><?php _e( 'Displays a link to the artist URL if possible, otherwise displays the artist name <sup>(8)</sup>',   GLASTFM_TEXT_DOMAIN ); ?></dd>

			<dt><strong>%coverurl%      </strong></dt><dd><?php _e( 'Cover URL <sup>(9)</sup>',          GLASTFM_TEXT_DOMAIN ); ?></dd>
			<dt><strong>%coversmallurl%  </strong></dt><dd><?php _e( 'Cover URL, small <sup>(9)</sup>',    GLASTFM_TEXT_DOMAIN ); ?></dd>
			<dt><strong>%covermediumurl%  </strong></dt><dd><?php _e( 'Cover URL, medium <sup>(9)</sup>',    GLASTFM_TEXT_DOMAIN ); ?></dd>
			<dt><strong>%coverlargeurl%    </strong></dt><dd><?php _e( 'Cover URL, large <sup>(9)</sup>',      GLASTFM_TEXT_DOMAIN ); ?></dd>
			<dt><strong>%coverextralargeurl%</strong></dt><dd><?php _e( 'Cover URL, extra-large <sup>(9)</sup>', GLASTFM_TEXT_DOMAIN ); ?></dd>
		</dl>
		<p><strong><?php _e( 'Notes:', GLASTFM_TEXT_DOMAIN ); ?></strong></p>
		<dl>
			<dt><strong>1:</strong></dt>
			<dd><?php _e( 'Recent Tracks, Loved Tracks, Top Tracks and Weekly Track Chart only', GLASTFM_TEXT_DOMAIN ); ?></dd>

			<dt><strong>2:</strong></dt>
			<dd><?php _e( 'Recent Tracks, Top Albums and Weekly Album Chart only', GLASTFM_TEXT_DOMAIN ); ?></dd>

			<dt><strong>3:</strong></dt>
			<dd><?php _e( 'Top Albums and Weekly Album Chart only', GLASTFM_TEXT_DOMAIN ); ?></dd>

			<dt><strong>4:</strong></dt>
			<dd><?php _e( 'Top Tracks, Top Artists, Top Albums, Loved Tracks and Weekly Artist Chart only', GLASTFM_TEXT_DOMAIN ); ?></dd>

			<dt><strong>5:</strong></dt>
			<dd><?php _e( 'Recent Tracks only', GLASTFM_TEXT_DOMAIN ); ?></dd>

			<dt><strong>6:</strong></dt>
			<dd><?php _e( 'Top Tracks, Top Artists, Top Albums, Weekly Track Chart and Weekly Album Chart only', GLASTFM_TEXT_DOMAIN ); ?></dd>

			<dt><strong>7:</strong></dt>
			<dd><?php _e( 'Recent Tracks: no link available', GLASTFM_TEXT_DOMAIN ); ?></dd>

			<dt><strong>8:</strong></dt>
			<dd><?php _e( 'Top Tracks and Loved Tracks: no link available', GLASTFM_TEXT_DOMAIN ); ?></dd>

			<dt><strong>9:</strong></dt>
			<dd><?php _e( 'Weekly artist chart and Weekly album chart: no pictures available', GLASTFM_TEXT_DOMAIN ); ?></dd>
		</dl>
		<p>
			<label for="<?php echo $this->get_field_id( 'caching_message' ); ?>"><?php _e( 'Display caching message:', GLASTFM_TEXT_DOMAIN ); ?></label>
			<input type="checkbox" id="<?php echo $this->get_field_id( 'caching_message' ); ?>" name="<?php echo $this->get_field_name( 'caching_message' ); ?>" value="on"<?php echo ( $instance['caching_message'] === TRUE ? ' checked="checked"' : '' ); ?> style="width:2em;" />
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'display_credits' ); ?>"><?php _e( 'Display credits:', GLASTFM_TEXT_DOMAIN ); ?></label>
			<input type="checkbox" id="<?php echo $this->get_field_id( 'display_credits' ); ?>" name="<?php echo $this->get_field_name( 'display_credits' ); ?>" value="on"<?php echo ( $instance['display_credits'] === TRUE ? ' checked="checked"' : '' ); ?> style="width:2em;" />
		</p>
		<p>
			<?php printf( __( '<a href="%s">Do not forget to set you API key in the settings!</a>', GLASTFM_TEXT_DOMAIN ), 'options-general.php?page=glastfm' ); ?>
		</p>
	<?php
	}
}
?>